/*
 * Handles word list selection form.
 */

#include <PalmOS.h>

#include "flipMain.h"
#include "flipDB.h"
#include "palmUtil.h"
#include "flipDicts.h"
#include "resdefs.h"


static Boolean DictEventHandler(EventPtr event);
static void DictFormInit(void);
static void DictFormDeinit(void);


struct FormInfo DictFormInfo = {
	EventStealer: NULL,
	EventHandler: DictEventHandler,
	InitForm: DictFormInit,
	DeinitForm: DictFormDeinit,
	UpdateForm: NULL,
};


static UInt16 returnTo;

extern void ShowDictForm(void)
{
	returnTo = FrmGetActiveFormID();
	FrmGotoForm(DictForm);
}

static void DictListDraw(Int16 itemNum, RectanglePtr bounds, Char **data)
{
	FlipDBInfo info;
	Err err;

	err = FlipGetDatabase(itemNum, &info);
	ErrNonFatalDisplayIf(err, "Error getting database info.");
	if (err)
		return;

	WinDrawChars(info.name, StrLen(info.name), bounds->topLeft.x,
		     bounds->topLeft.y);
}

static void DictListInit(void)
{
	UInt16 dbs;
	Err err;
	ListPtr list = GetObjectPtr(DictFormList);
	UInt16 sel;
	FlipDBInfo openDBInfo;

	err = FlipNumDatabases(&dbs);
	ErrNonFatalDisplayIf(err, "Error getting number of databases.");
	if (err)
		return;
	LstSetListChoices(list, NULL, dbs);

	err = FlipGetOpenDatabase(wordDB, &openDBInfo);
	ErrNonFatalDisplayIf(err,
			     "Error getting info about open database.");
	if (!err) {
		err = FlipGetDatabaseIndex(&sel, &openDBInfo);
		ErrNonFatalDisplayIf(err,
				     "Error getting index of open database.");
		if (!err)
			LstSetSelection(list, sel);
	}

	LstSetDrawFunction(list, DictListDraw);
}

static void DictListDeinit(void)
{

}

static void DictFormInit(void)
{
	DictListInit();
}

static void DictFormDeinit(void)
{
	DictListDeinit();
}

static void DictSwitchToSelected(void)
{
	FlipDBInfo info;
	ListPtr list = GetObjectPtr(DictFormList);
	UInt16 sel;

	sel = LstGetSelection(list);
	FlipGetDatabase(sel, &info);
	SwitchDB(&wordDB, &info);
}

extern Boolean DictEventHandler(EventPtr event)
{
	switch (event->eType) {
	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {
		case DictFormSelectButton:
			DictSwitchToSelected();
			FrmGotoForm(returnTo);
			return true;
		}
		return false;

	default:
		return false;
	}
}
